"""Subclonal composition inference"""

import pickle

import logbook
import numpy
import pandas
import scipy.special
import scipy.optimize
import scipy.signal
import scipy.stats
from itertools import chain


_LOG = logbook.Logger(__name__)


_MINIMAL_MUTATIONS = 50

_SMALLEST_CLONE = 10

_MINIMAL_SUBCLONE = 1e-7


def infer_single_sample(mutations, purity):
    """Infer subclones for a single sample.

    Parameters
    ----------
    mutations : pandas.DataFrame
        The mutation table.
    purity : float | str | NoneType
        The estimated tumour purity.

    Returns
    -------
    numpy.ndarray[float]
        The subclone table.
    numpy.ndarray[float]
        The mutation assignment to subclones.
    """
    if isinstance(purity, str):
        peaks = numpy.asarray([float(x) for x in purity.split(':')])
        return _assign_mutations_to_subclones(mutations, peaks)
    filter_ = ((mutations['major_copy1'] == mutations['minor_copy1']) &
               (mutations['state1'] == 1.0))
    if filter_.sum() < _MINIMAL_MUTATIONS:
        filter_ = mutations['state1'] == 1.0
    if filter_.sum() > _MINIMAL_MUTATIONS:
        peaks = _infer_without_2state_cna(mutations[filter_], purity)
    else:
        peaks = _infer_with_2state_cna(mutations, purity)
    return _assign_mutations_to_subclones(mutations, peaks)


def _infer_without_2state_cna(mutations, purity):
    """Infer subclones for a single sample.

    Parameters
    ----------
    mutations : pandas.DataFrame
        The mutation table.
    purity : float | NoneType
        The estimated tumour purity.

    Returns
    -------
    numpy.ndarray[float]
        The subclone cell prevalance.
    """
    r = 1.0 if purity is None else purity
    f = mutations['allelic_count'] / mutations['total_count']
    ratios = f * ((mutations['major_copy1'] + mutations['minor_copy1']) * r
                  + mutations['normal_copy'] * (1 - r))
    peaks = _get_density_peaks(ratios)
    if purity is not None:
        peaks = _adjust_subclone_on_purity(peaks, purity)
    else:
        if peaks[-1] >= 1.1:
            peaks = [p for p in peaks if p < 1.1]
        if peaks[-1] >= 1:
            peaks = [p for p in peaks if p < 1.0] + [1.0]
        r = peaks[-1]
        ratios = f * ((mutations['major_copy1'] + mutations['minor_copy1']) * r
                      + mutations['normal_copy'] * (1 - r))
        peaks = _get_density_peaks(ratios)
        peaks = _adjust_subclone_on_purity(peaks, r)
    return numpy.asarray(peaks)


def _infer_with_2state_cna(mutations, purity):
    """Infer subclones for a single sample.

    Parameters
    ----------
    mutations : pandas.DataFrame
        The mutation table.
    purity : float | NoneType
        The estimated tumour purity.

    Returns
    -------
    numpy.ndarray[float]
        The subclone cell prevalance.
    """
    r = 1.0 if purity is None else purity
    f = mutations['allelic_count'] / mutations['total_count']
    ratios = []
    for i in range(len(mutations)):
        tmp_ratios = [f[i] * ((mj_copy + mutations['minor_copy1'].iloc[i]) * r + mutations['normal_copy'].iloc[i] * (1 - r))
                      for mj_copy in range(int(mutations['major_copy1'].iloc[i]))]
        ratios.append(tmp_ratios)
    ratios = list(chain(*ratios))
    ratios = pandas.Series(ratios)
    peaks = _get_density_peaks(ratios)
    if purity is not None:
        peaks = _adjust_subclone_on_purity(peaks, purity)
    else:
        if peaks[-1] >= 1.1:
            peaks = [p for p in peaks if p < 1.1]
        if peaks[-1] >= 1:
            peaks = [p for p in peaks if p < 1.0] + [1.0]
        r = peaks[-1]
        ratios = f * ((mutations['major_copy1'] + mutations['minor_copy1']) * r
                      + mutations['normal_copy'] * (1 - r))
        peaks = _get_density_peaks(ratios)
        peaks = _adjust_subclone_on_purity(peaks, r)
    return numpy.asarray(peaks)


def _get_density_peaks(samples, grid_count=200):
    """Identify density peaks

    Parameters
    ----------
    samples : numpy.ndarray[float]
        The samples for estimating density.
    grid_count : int
        The number of data point for scanning maxima. By default, it is 200.

    Returns
    -------
    list[float]
        The subclone cell prevalance.
    """
    kde = scipy.stats.gaussian_kde(samples.values.flatten())
    delta = 0.5 / grid_count
    x = numpy.linspace(delta, 1.0 - delta, grid_count)
    logpdf = kde.logpdf(x)
    maxidx = list(scipy.signal.argrelmax(logpdf)[0])
    if logpdf[0] > logpdf[1]:
        maxidx.append(0)
    if logpdf[-1] > logpdf[-2]:
        maxidx.append(len(logpdf) - 1)
    peaks = [float(scipy.optimize.fminbound(lambda x: -kde.logpdf(x),
                                            x[idx] - delta, x[idx] + delta,
                                            xtol=1e-6, full_output=False,
                                            disp=False))
             for idx in maxidx]
    peaks.sort()
    _LOG.info('Found peaks: {}', peaks)
    return peaks


def _adjust_subclone_on_purity(peaks, purity):
    """Adjust peaks based on tumour purity.

    Parameters
    ----------
    peaks : list[float]
        The identified density peaks of subclones.
    purity : float | NoneType
        The estimated tumour purity.

    Returns
    -------
    list[float]
        The adjusted density peaks of subclones
    """
    if purity is not None:
        max_frac = purity
        diff = numpy.abs(numpy.asarray(peaks) - max_frac)
        max_peak_idx = diff.argmin()
        if diff[max_peak_idx] > 0.1:
            peaks = peaks + [max_frac]
        else:
            peaks = peaks[:max_peak_idx + 1]
    if peaks[-1] >= 1.1:
        peaks = [p for p in peaks if p < 1.1]
    if peaks[-1] >= 1:
        peaks = [p for p in peaks if p < 1.0] + [1.0]
    _LOG.info('Adjusted peaks to: {}', peaks)
    return peaks


def _assign_mutations_to_subclones(mutations, peaks):
    """Assign mutations to subclones.

    Parameters
    ----------
    mutations : pandas.DataFrame
        The mutations for estimating subclone weights.
    peaks : list[float]
        The identified density peaks of subclones.

    Returns
    -------
    numpy.ndarray[int]
        The adjusted subclones for all mutations.
    numpy.ndarray[float]
        The probability of mutations associated to subclones.
    """
    _LOG.debug('Found raw subclone prevalences: {}', peaks)
    while True:
        weights, scores = _estimate_subclone_weights(mutations, peaks)
        assignment = scores.values.argmax(axis=1)
        idx, size = numpy.unique(assignment, return_counts=True)
        idx = idx[size >= _SMALLEST_CLONE]
        if idx.size == peaks.size or len(mutations) < _MINIMAL_MUTATIONS * 2:
            return peaks, scores
        peaks = peaks[idx]


def _estimate_subclone_weights(mutations, peaks):
    """Estimate subclone weights.

    Parameters
    ----------
    mutations : pandas.DataFrame
        The mutations for estimating subclone weights.
    peaks : list[float]
        The identified density peaks of subclones.

    Returns
    -------
    numpy.ndarray[float]
        The weights of subclones.
    numpy.ndarray[int]
        The assignment of mutations to subclones.
    """
    peaks = peaks[None, :, None]
    purity = peaks.max()
    weights = numpy.repeat(1 / peaks.size, peaks.size).astype('f4')
    major = mutations['major_copy1'].values[:, None, None]
    minor = mutations['minor_copy1'].values[:, None, None]
    state1 = mutations['state1'].values[:, None, None]
    denominator = state1 * purity * (major + minor) + (1 - state1 * purity) * 2
    numerator = numpy.dstack([
        numpy.where(peaks < state1, 0, (major * state1 * purity +
                                        peaks - state1 * purity)),
        numpy.where(peaks < state1, 0, (minor * state1 * purity +
                                        peaks - state1 * purity)),
        numpy.repeat(numpy.arange(1, (numpy.nanmax(major) + 1)) * peaks, len(mutations), axis=0)
    ])
    copy_number_category = numpy.unique(major)
    for copy_number in copy_number_category:
        numerator[numpy.argwhere(major == copy_number)[0][0]][0][int(copy_number + 2):] = 0
    p = numerator / denominator
    x = mutations['allelic_count'].values[:, None, None]
    k = mutations['total_count'].values[:, None, None]
    l = numpy.nanmax(scipy.stats.binom.logpmf(x, k, p), axis=2) + numpy.log(weights)
    l_sample = scipy.special.logsumexp(l, axis=1)
    l_sum = numpy.nansum(l_sample)
    for t in range(5000):
        old_l_sum = l_sum
        scores = numpy.exp(l - l_sample[:, None])
        weights = numpy.nanmean(scores, axis=0)
        weights /= weights.sum()
        l = numpy.nanmax(scipy.stats.binom.logpmf(x, k, p), axis=2) + numpy.log(weights)
        l_sample = scipy.special.logsumexp(l, axis=1)
        l_sum = numpy.nansum(l_sample)
        if l_sum > old_l_sum and l_sum < old_l_sum + 0.001:
            _LOG.info('EM is fully converged.')
            break
    else:
        _LOG.warning('Maximum iteration reached but the EM did not converged.')
    scores = pandas.DataFrame(numpy.exp(l - l_sample[:, None]),
                              index=mutations.index, dtype='f4')
    return weights, scores



import pathlib
import sys

import fire
import logbook
import pandas

from . import loaders
from . import phylogeny
from . import subclone


_LOG = logbook.Logger(__name__)


class Entrypoint:
    """FastClone: a tumour subclone inference tool

    Check subcommands for sample loading.
    Chain multiple subcommands load multiple samples and infer.
    Use '-- --help' to show help messages.

    Examples
    --------
    fastclone load-pyclone t1 t1.tsv None load-pyclone t2 t2.tsv 0.8 solve

    Notes
    -----
    If MuTect VCF and PyClone samples are provided, note that MuTect
    mutations are labelled as 'Chromosome:Coordinate:AltBase', such as
    'Y:15989697:G'. Make sure PyClone ID uses the same ID.
    """

    def __init__(self):
        self._samples = dict()

    def load_mutect_battenberg(self, name, vcf_path, vcf_sample,
                               battenberg_path):
        """Load a sample from MuTect and Battenberg files.

        Parameters
        ----------
        name : str
            The name of the sample.
        vcf_path : str
            The MuTect VCF file.
        vcf_sample : str
            The column of sample in the VCF file to read.
        battenberg_path : str
            The Battenberg file.
        """
        if name in self._samples:
            raise ValueError('duplicate sample: ' + name)
        _LOG.info('Loading {} ...', name)
        _LOG.info('VCF file {} (column: {})', vcf_path, vcf_sample)
        _LOG.info('Battenberg file {}', battenberg_path)
        self._samples[name] = loaders.load_mutect_and_battenberg(
            vcf_path, vcf_sample, battenberg_path,
        )
        return self

    def load_pyclone(self, name, path, purity):
        """Load a sample from a PyClone file.

        Parameters
        ----------
        path : str
            A path or a file handle of the VCF file.
        purity : float
            An estimated tumour turity. If unknown, use None.
        """
        if name in self._samples:
            raise ValueError('duplicate sample: ' + name)
        _LOG.info('Loading {} ...', name)
        _LOG.info('PyClone file {}', path)
        if purity is None:
            _LOG.info('Infer the purity later.')
        else:
            _LOG.info('Purity: {}', purity)
        self._samples[name] = (loaders.load_pyclone(path), purity)
        return self

    def load_pyclone_truth(self, name, path, truth):
        """Load a sample from a PyClone file with pickled data.

        Parameters
        ----------
        path : str
            A path or a file handle of the VCF file.
        truth : str
            A truth subclonal frequency file.
        """
        if name in self._samples:
            raise ValueError('duplicate sample: ' + name)
        _LOG.info('Loading {} ...', name)
        _LOG.info('PyClone file {}', path)
        _LOG.info('Truth file: {}', truth)
        self._samples[name] = (loaders.load_pyclone(path), truth)
        return self

    def solve(self, output):
        """Infer the subclonal composition.

        Parameters
        ----------
        output : str
            The output path.
        """
        if not self._samples:
            raise RuntimeError('inference needs at least one sample')
        if len(self._samples) > 1:
            raise NotImplementedError('mult-sample inference is not available')
        _LOG.info('Solving subclonal composition ...')
        pathlib.Path(output).mkdir(parents=True, exist_ok=True)
        samples = list(self._samples.items())
        subclones = None
        for name, sample in samples:
            sample_subclones, score = subclone.infer_single_sample(*sample)
            sample_subclones = pandas.DataFrame(sample_subclones,
                                                columns=[name])
            if subclones is None:
                subclones = sample_subclones
        phylogeny.infer(subclones, score, output)
        pandas.DataFrame(subclones).to_csv(output + '/subclones.csv')
        pandas.DataFrame(score).to_csv(output + '/scores.csv')
    
    @staticmethod
    def version():
        """Display the FastClone version.
        """
        
        print('Ver. 1.0.8')


def main():
    with logbook.StderrHandler(level="INFO").applicationbound():
        fire.Fire(Entrypoint)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

import setuptools

with open('README.md', 'r') as fh:
        long_description = fh.read()

setuptools.setup(
    name='fastclone-guanlab',
    version='1.0.8',
    description='An inference tool for tumour subclonal composition',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Hongjiu Zhang, Yuanfang Guan, Yao Xiao',
    author_email='zhanghj@umich.edu, gyuanfan@umich.edu, yxiaoy@umich.edu',
    url='http://github.com/GuanLab/FastClone_GuanLab',
    license='GPLv3+',
    keywords='bioinformatics',
    python_requires='>=3.5.0',
    packages=setuptools.find_packages(exclude=['tests']),
    entry_points={
        'console_scripts': [
            'fastclone = fastclone.__main__:main',
        ],
    },
    package_data={
        'fastclone': ['test/data/*'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        ('License :: OSI Approved '
         ':: GNU General Public License v3 or later (GPLv3+)'),
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
    #setup_requires=['pytest-runner'],
    #tests_require=['pytest', 'pytest-datadir'],
    install_requires=['logbook', 'pandas', 'scikit-learn', 'fire', 'networkx', 'matplotlib'],
)



"""Subclonal phylogeny inference"""

import networkx as nx
import random
import matplotlib.pyplot as plt
from collections import Counter
import numpy as np

def infer(subclones, score, output):
    G = nx.Graph()

    if len(subclones) > 1:
        edge_ls = []
        root = len(subclones) - 1
        
        snv_assignments = score.values.argmax(axis=1)
        subclone_snv_number = Counter(snv_assignments)
        
        child_parent = {root:root}
        tmp_root = root
        for i in range(len(subclones) - 1):
            i_inverse = root - i - 1
            while True:
                if subclone_snv_number[i_inverse] < subclone_snv_number[tmp_root]:
                    edge_ls.append([tmp_root, i_inverse])
                    child_parent[i_inverse] = tmp_root
                    tmp_root = i_inverse
                    break
                else:
                    if tmp_root == root:
                        edge_ls.append([tmp_root, i_inverse])
                        child_parent[i_inverse] = tmp_root
                        break
                    tmp_root = child_parent[tmp_root]
                 
                    

        G.add_edges_from(edge_ls)
        
        child_in_parent_prop = list()
        for edge in edge_ls:
            p = subclones.iloc[edge[1], 0]/subclones.iloc[edge[0], 0]
            edge.append(p)
            child_in_parent_prop.append(edge)

        pd.DataFrame(child_in_parent_prop, columns=['Parent', 'Child', 'The proportion of child clone']).to_csv(output + '/phylogeny_proportion.csv')

    else:
        G.add_nodes_from([0])

    pos = hierarchy_pos(G, root=len(subclones) - 1)
    nx.draw(G, pos=pos, with_labels=True, node_color='orange', alpha=0.7)
    plt.savefig(output + '/phylogeny.png')


def hierarchy_pos(G, root=None, width=1., vert_gap = 0.2, vert_loc = 0, xcenter = 0.5):
    if not nx.is_tree(G):
        raise TypeError('cannot use hierarchy_pos on a graph that is not a tree')

    if root is None:
        if isinstance(G, nx.DiGraph):
            root = next(iter(nx.topological_sort(G)))  #allows back compatibility with nx version 1.11
        else:
            root = random.choice(list(G.nodes))

    def _hierarchy_pos(G, root, width=1., vert_gap = 0.2, vert_loc = 0, xcenter = 0.5, pos = None, parent = None):
        '''
        see hierarchy_pos docstring for most arguments

        pos: a dict saying where all nodes go if they have been assigned
        parent: parent of this branch. - only affects it if non-directed

        '''

        if pos is None:
            pos = {root:(xcenter,vert_loc)}
        else:
            pos[root] = (xcenter, vert_loc)
        children = list(G.neighbors(root))
        if not isinstance(G, nx.DiGraph) and parent is not None:
            children.remove(parent)
        if len(children)!=0:
            dx = width/len(children)
            nextx = xcenter - width/2 - dx/2
            for child in children:
                nextx += dx
                pos = _hierarchy_pos(G,child, width = dx, vert_gap = vert_gap,
                                    vert_loc = vert_loc-vert_gap, xcenter=nextx,
                                    pos=pos, parent = root)
        return pos


    return _hierarchy_pos(G, root, width, vert_gap, vert_loc, xcenter)



"""Various input loaders."""


import pandas


__all__ = (
    'load_mutect_and_battenberg',
    'load_pyclone',
)


def load_mutect_and_battenberg(vcf_path, vcf_sample, battenberg_path):
    """Load SNVs and CNAs from MuTect VCF and Battenberg files.

    The file format is used in the DREAM SMC-Het Challenge.

    Parameters
    ----------
    vcf_path : file-like object, pathlib.Path or str
        A path or a file handle of the VCF file.
    vcf_sample : str
        The column of sample in the VCF file to read.
    battenberg_path : file-like object, pathlib.Path or str
        A path or a file handle of the Battenberg file.

    Returns
    -------
    pandas.DataFrame
        The mutation data.
    float | NoneType
        The estimated purity.
    """
    vcf = pandas.read_csv(vcf_path, comment='#', sep='\t', header=None)
    vcf.columns = _read_mutect_vcf_columns(vcf_path)
    data = pandas.DataFrame()
    data['chromosome'] = vcf['CHROM'].astype(str)
    data['coordinate'] = vcf['POS']
    data['base'] = vcf['ALT']
    sample = vcf[vcf_sample].str.split(':', expand=True)
    
    try:
        read_counts = sample[1].str.split(',', expand=True).astype('i4')
        data['total_count'] = read_counts[0] + read_counts[1]
        data['allelic_count'] = read_counts[1]
    except ValueError:
        data['total_count'] = [100] * len(sample[4])
        data['allelic_count'] = 100 * sample[4].astype(float)
        
    data['normal_copy'] = 2
    data['major_copy1'] = 1
    data['minor_copy1'] = 1
    data['major_copy2'] = 1
    data['minor_copy2'] = 1
    data['state1'] = 1.0
    if (data['chromosome'] == 'Y').any():
        data.loc[data['chromosome'] == 'X', 'normal_copy'] = 1
        data.loc[data['chromosome'] == 'Y', 'normal_copy'] = 1
        data.loc[data['chromosome'] == 'X', 'minor_copy1'] = 0
        data.loc[data['chromosome'] == 'Y', 'minor_copy1'] = 0
        data.loc[data['chromosome'] == 'X', 'minor_copy2'] = 0
        data.loc[data['chromosome'] == 'Y', 'minor_copy2'] = 0
    purity = _load_battenberg_cnas(data, battenberg_path)
    data.index = (data['chromosome'] + ':' + data['coordinate'].astype(str)
                  + ':' + data['base'])
    data.drop(['chromosome', 'coordinate', 'base'], axis=1, inplace=True)
    data = data[data.allelic_count/data.total_count > 0.01]
    return data, purity


def _read_mutect_vcf_columns(vcf_path, add_smc_het_column=False):
    """Read MuTect VCF column names.

    Parameters
    ----------
    vcf_path : file-like object, pathlib.Path or str
        A path or a file handle of the VCF file.
    add_smc_het_column : bool
        Whether to add a column at the end for the somatic mutation
        calling truth. This is only for SMC-Het challenge truth files.
        It is disabled by default.

    Returns
    -------
    list[str]
        List of VCF column names.
    """
    for line in open(vcf_path):
        if line[1] != '#' and line[0] == '#':
            columns = line[1:].strip().split()
            if add_smc_het_column:
                columns.append(None)
            return columns
    raise RuntimeError('VCF header line not found')


def _load_battenberg_cnas(data, path):
    """Load CNA information from the Battenberg file.

    Update the mutation data with new copy number data. Also, estimate
    the purity of the tumour tissue.

    Parameters
    ----------
    data : pandas.DataFrame
        The mutation data.
    path : file-like object, pathlib.Path or str
        A path or a file handle of the Battenberg file.

    Returns
    -------
    float
        The estimated purity of the tumour tissue.
    """
    table = pandas.read_csv(path, sep='\t')
    table['chr'] = table['chr'].astype(str)
    if 'Y' in table['chr']:
        data.loc[data['chromosome'] == 'X', 'normal_copy'] = 1
        data.loc[data['chromosome'] == 'Y', 'normal_copy'] = 1
    purities = []
    for __, row in table.iterrows():
        purity = _estimate_purity_from_battenberg(row)
        if purity is not None:
            purities.append(purity)
        filter_ = ((data['chromosome'] == row['chr'])
                   & (data['coordinate'] > row['startpos'])
                   & (data['coordinate'] <= row['endpos']))
        if row['frac1_A'] == 1.0:
            data.loc[filter_, 'major_copy1'] = int(row['nMaj1_A'])
            data.loc[filter_, 'minor_copy1'] = int(row['nMin1_A'])
        else:
            data.loc[filter_, 'major_copy1'] = int(row['nMaj1_A'])
            data.loc[filter_, 'minor_copy1'] = int(row['nMin1_A'])
            data.loc[filter_, 'state1'] = row['frac1_A']
            data.loc[filter_, 'major_copy2'] = int(row['nMaj2_A'])
            data.loc[filter_, 'minor_copy2'] = int(row['nMin2_A'])
    if purities:
        return sum(purities) / len(purities)
    else:
        return None


def _estimate_purity_from_battenberg(entry):
    """Estimate the tumour purity from the Battenberg entry.

    Parameters
    ----------
    entry : pandas.Series
        The Battenberg copy number entry.

    Returns
    -------
    float | NoneType
        The estimated purity of the tumour tissue.
    """
    if ((entry['frac1_A'] == 1.0 and entry['nMaj1_A'] == entry['nMin1_A'])
            or (entry['frac1_A'] != 1.0
                and entry['nMaj1_A'] == entry['nMin1_A']
                and entry['nMaj2_A'] == entry['nMin2_A'])):
        return None
    if entry['frac1_A'] == 1.0:
        return ((1 - 2 * entry["BAF"])
                / (1 + entry["BAF"] * (entry["nMaj1_A"] + entry["nMin1_A"])
                   - 2 * entry["BAF"] - entry["nMaj1_A"]))
    return ((1 - 2 * entry["BAF"])
            / (1 + entry["BAF"] * (entry["frac1_A"] * (entry["nMaj1_A"]
                                                       + entry["nMin1_A"])
                                   + entry["frac2_A"] * (entry["nMaj2_A"]
                                                         + entry["nMin2_A"]))
               - 2 * entry["BAF"] - entry["nMaj1_A"] * entry["frac1_A"]
               - entry["nMaj2_A"] * entry["frac2_A"]))


def load_pyclone(path):
    """Load SNVs and CNAs from the PyClone file.

    Parameters
    ----------
    path : file-like object, pathlib.Path or str
        A path or a file handle of the PyClone input file.
    battenberg_path : file-like object, pathlib.Path or str
        A path or a file handle of the Battenberg file.

    Returns
    -------
    pandas.DataFrame
        The mutation data.
    """
    data = pandas.read_csv(path, sep='\t')[[
        'mutation_id', 'ref_counts', 'var_counts', 'normal_cn',
        'major_cn', 'minor_cn',
    ]]
    data.set_index('mutation_id', inplace=True)
    data.index.name = None
    data['ref_counts'] += data['var_counts']
    data.columns = ['total_count', 'allelic_count', 'normal_copy',
                    'major_copy1', 'minor_copy1']
    data['major_copy2'] = 1
    data['minor_copy2'] = 1
    data['state1'] = 1.0
    data = data[data.allelic_count/data.total_count > 0.01]
    return data
